@@ -10,6 +10,7 @@
 // the appropriate sources are ready, the program is run as:
 //     go run linux/mkall.go <linux_dir> <glibc_dir>
 
+//go:build ignore
 // +build ignore
 
 package main
@@ -21,6 +22,7 @@ import (
 	"encoding/binary"
 	"errors"
 	"fmt"
+	"go/build/constraint"
 	"io"
 	"io/ioutil"
 	"os"
@@ -28,6 +30,7 @@ import (
 	"path/filepath"
 	"runtime"
 	"strings"
+	"sync"
 	"unicode"
 )
 
@@ -36,8 +39,6 @@ var LinuxDir string
 var GlibcDir string
 
 const TempDir = "/tmp"
-const IncludeDir = TempDir + "/include" // To hold our C headers
-const BuildDir = TempDir + "/build"     // To hold intermediate build files
 
 const GOOS = "linux"       // Only for Linux targets
 const BuildArch = "amd64"  // Must be built on this architecture
@@ -50,11 +51,14 @@ type target struct {
 	BigEndian  bool   // Default Little Endian
 	SignedChar bool   // Is -fsigned-char needed (default no)
 	Bits       int
+	env        []string
+	stderrBuf  bytes.Buffer
+	compiler   string
 }
 
-// List of all Linux targets supported by the go compiler. Currently, riscv64
-// and sparc64 are not fully supported, but there is enough support already to
-// generate Go type and error definitions.
+// List of all Linux targets supported by the go compiler. Currently, sparc64 is
+// not fully supported, but there is enough support already to generate Go type
+// and error definitions.
 var targets = []target{
 	{
 		GoArch:    "386",
@@ -81,6 +85,12 @@ var targets = []target{
 		GNUArch:   "arm-linux-gnueabi",
 		Bits:      32,
 	},
+	{
+		GoArch:    "loong64",
+		LinuxArch: "loongarch",
+		GNUArch:   "loongarch64-linux-gnu",
+		Bits:      64,
+	},
 	{
 		GoArch:    "mips",
 		LinuxArch: "mips",
@@ -107,6 +117,13 @@ var targets = []target{
 		GNUArch:   "mips64el-linux-gnuabi64",
 		Bits:      64,
 	},
+	{
+		GoArch:    "ppc",
+		LinuxArch: "powerpc",
+		GNUArch:   "powerpc-linux-gnu",
+		BigEndian: true,
+		Bits:      32,
+	},
 	{
 		GoArch:    "ppc64",
 		LinuxArch: "powerpc",
@@ -178,13 +195,44 @@ func main() {
 	LinuxDir = os.Args[1]
 	GlibcDir = os.Args[2]
 
+	wg := sync.WaitGroup{}
 	for _, t := range targets {
-		fmt.Printf("----- GENERATING: %s -----\n", t.GoArch)
-		if err := t.generateFiles(); err != nil {
-			fmt.Printf("%v\n***** FAILURE:    %s *****\n\n", err, t.GoArch)
-		} else {
-			fmt.Printf("----- SUCCESS:    %s -----\n\n", t.GoArch)
+		fmt.Printf("arch %s: GENERATING\n", t.GoArch)
+		if err := t.setupEnvironment(); err != nil {
+			fmt.Printf("arch %s: could not setup environment: %v\n", t.GoArch, err)
+			break
 		}
+		includeDir := filepath.Join(TempDir, t.GoArch, "include")
+		// Make the include directory and fill it with headers
+		if err := os.MkdirAll(includeDir, os.ModePerm); err != nil {
+			fmt.Printf("arch %s: could not make directory: %v\n", t.GoArch, err)
+			break
+		}
+		// During header generation "/git/linux/scripts/basic/fixdep" is created by "basic/Makefile" for each
+		// instance of "make headers_install". This leads to a "text file is busy" error from any running
+		// "make headers_install" after the first one's target. Workaround is to serialize header generation
+		if err := t.makeHeaders(); err != nil {
+			fmt.Printf("arch %s: could not make header files: %v\n", t.GoArch, err)
+			break
+		}
+		wg.Add(1)
+		go func(t target) {
+			defer wg.Done()
+			fmt.Printf("arch %s: header files generated\n", t.GoArch)
+			if err := t.generateFiles(); err != nil {
+				fmt.Printf("%v\n***** FAILURE:    %s *****\n\n", err, t.GoArch)
+			} else {
+				fmt.Printf("arch %s: SUCCESS\n", t.GoArch)
+			}
+		}(t)
+	}
+	wg.Wait()
+
+	fmt.Printf("----- GENERATING: merging generated files -----\n")
+	if err := mergeFiles(); err != nil {
+		fmt.Printf("%v\n***** FAILURE:    merging generated files *****\n\n", err)
+	} else {
+		fmt.Printf("----- SUCCESS:    merging generated files -----\n\n")
 	}
 
 	fmt.Printf("----- GENERATING ptrace pairs -----\n")
@@ -205,17 +253,29 @@ func main() {
 	}
 }
 
-// Makes an exec.Cmd with Stderr attached to os.Stderr
-func makeCommand(name string, args ...string) *exec.Cmd {
+func (t *target) printAndResetBuilder() {
+	if t.stderrBuf.Len() > 0 {
+		for _, l := range bytes.Split(t.stderrBuf.Bytes(), []byte{'\n'}) {
+			fmt.Printf("arch %s: stderr: %s\n", t.GoArch, l)
+		}
+		t.stderrBuf.Reset()
+	}
+}
+
+// Makes an exec.Cmd with Stderr attached to the target string Builder, and target environment
+func (t *target) makeCommand(name string, args ...string) *exec.Cmd {
 	cmd := exec.Command(name, args...)
-	cmd.Stderr = os.Stderr
+	cmd.Env = t.env
+	cmd.Stderr = &t.stderrBuf
 	return cmd
 }
 
 // Set GOARCH for target and build environments.
 func (t *target) setTargetBuildArch(cmd *exec.Cmd) {
 	// Set GOARCH_TARGET so command knows what GOARCH is..
-	cmd.Env = append(os.Environ(), "GOARCH_TARGET="+t.GoArch)
+	var env []string
+	env = append(env, t.env...)
+	cmd.Env = append(env, "GOARCH_TARGET="+t.GoArch)
 	// Set GOARCH to host arch for command, so it can run natively.
 	for i, s := range cmd.Env {
 		if strings.HasPrefix(s, "GOARCH=") {
@@ -227,23 +287,31 @@ func (t *target) setTargetBuildArch(cmd *exec.Cmd) {
 // Runs the command, pipes output to a formatter, pipes that to an output file.
 func (t *target) commandFormatOutput(formatter string, outputFile string,
 	name string, args ...string) (err error) {
-	mainCmd := makeCommand(name, args...)
+	mainCmd := t.makeCommand(name, args...)
 	if name == "mksyscall" {
 		args = append([]string{"run", "mksyscall.go"}, args...)
-		mainCmd = makeCommand("go", args...)
+		mainCmd = t.makeCommand("go", args...)
 		t.setTargetBuildArch(mainCmd)
 	} else if name == "mksysnum" {
 		args = append([]string{"run", "linux/mksysnum.go"}, args...)
-		mainCmd = makeCommand("go", args...)
+		mainCmd = t.makeCommand("go", args...)
 		t.setTargetBuildArch(mainCmd)
 	}
 
-	fmtCmd := makeCommand(formatter)
+	fmtCmd := t.makeCommand(formatter)
 	if formatter == "mkpost" {
-		fmtCmd = makeCommand("go", "run", "mkpost.go")
+		fmtCmd = t.makeCommand("go", "run", "mkpost.go")
 		t.setTargetBuildArch(fmtCmd)
+	} else if formatter == "gofmt2" {
+		fmtCmd = t.makeCommand("gofmt")
+		mainCmd.Dir = filepath.Join(TempDir, t.GoArch, "mkerrors")
+		if err = os.MkdirAll(mainCmd.Dir, os.ModePerm); err != nil {
+			return err
+		}
 	}
 
+	defer t.printAndResetBuilder()
+
 	// mainCmd | fmtCmd > outputFile
 	if fmtCmd.Stdin, err = mainCmd.StdoutPipe(); err != nil {
 		return
@@ -266,20 +334,19 @@ func (t *target) commandFormatOutput(formatter string, outputFile string,
 	return mainCmd.Run()
 }
 
-// Generates all the files for a Linux target
-func (t *target) generateFiles() error {
+func (t *target) setupEnvironment() error {
 	// Setup environment variables
-	os.Setenv("GOOS", GOOS)
-	os.Setenv("GOARCH", t.GoArch)
+	t.env = append(os.Environ(), fmt.Sprintf("%s=%s", "GOOS", GOOS))
+	t.env = append(t.env, fmt.Sprintf("%s=%s", "GOARCH", t.GoArch))
 
 	// Get appropriate compiler and emulator (unless on x86)
 	if t.LinuxArch != "x86" {
 		// Check/Setup cross compiler
-		compiler := t.GNUArch + "-gcc"
-		if _, err := exec.LookPath(compiler); err != nil {
+		t.compiler = t.GNUArch + "-gcc"
+		if _, err := exec.LookPath(t.compiler); err != nil {
 			return err
 		}
-		os.Setenv("CC", compiler)
+		t.env = append(t.env, fmt.Sprintf("%s=%s", "CC", t.compiler))
 
 		// Check/Setup emulator (usually first component of GNUArch)
 		qemuArchName := t.GNUArch[:strings.Index(t.GNUArch, "-")]
@@ -288,76 +355,83 @@ func (t *target) generateFiles() error {
 		}
 		// Fake uname for QEMU to allow running on Host kernel version < 4.15
 		if t.LinuxArch == "riscv" {
-			os.Setenv("QEMU_UNAME", "4.15")
+			t.env = append(t.env, fmt.Sprintf("%s=%s", "QEMU_UNAME", "4.15"))
 		}
-		os.Setenv("GORUN", "qemu-"+qemuArchName)
+		t.env = append(t.env, fmt.Sprintf("%s=%s", "GORUN", "qemu-"+qemuArchName))
 	} else {
-		os.Setenv("CC", "gcc")
+		t.compiler = "gcc"
+		t.env = append(t.env, fmt.Sprintf("%s=%s", "CC", "gcc"))
 	}
+	return nil
+}
 
-	// Make the include directory and fill it with headers
-	if err := os.MkdirAll(IncludeDir, os.ModePerm); err != nil {
-		return err
-	}
-	defer os.RemoveAll(IncludeDir)
-	if err := t.makeHeaders(); err != nil {
-		return fmt.Errorf("could not make header files: %v", err)
-	}
-	fmt.Println("header files generated")
-
+// Generates all the files for a Linux target
+func (t *target) generateFiles() error {
 	// Make each of the four files
 	if err := t.makeZSysnumFile(); err != nil {
 		return fmt.Errorf("could not make zsysnum file: %v", err)
 	}
-	fmt.Println("zsysnum file generated")
+	fmt.Printf("arch %s: zsysnum file generated\n", t.GoArch)
 
 	if err := t.makeZSyscallFile(); err != nil {
 		return fmt.Errorf("could not make zsyscall file: %v", err)
 	}
-	fmt.Println("zsyscall file generated")
+	fmt.Printf("arch %s: zsyscall file generated\n", t.GoArch)
 
 	if err := t.makeZTypesFile(); err != nil {
 		return fmt.Errorf("could not make ztypes file: %v", err)
 	}
-	fmt.Println("ztypes file generated")
+	fmt.Printf("arch %s: ztypes file generated\n", t.GoArch)
 
 	if err := t.makeZErrorsFile(); err != nil {
 		return fmt.Errorf("could not make zerrors file: %v", err)
 	}
-	fmt.Println("zerrors file generated")
+	fmt.Printf("arch %s: zerrors file generated\n", t.GoArch)
 
 	return nil
 }
 
 // Create the Linux, glibc and ABI (C compiler convention) headers in the include directory.
 func (t *target) makeHeaders() error {
+	defer t.printAndResetBuilder()
+
 	// Make the Linux headers we need for this architecture
-	linuxMake := makeCommand("make", "headers_install", "ARCH="+t.LinuxArch, "INSTALL_HDR_PATH="+TempDir)
+	linuxMake := t.makeCommand("make", "headers_install", "ARCH="+t.LinuxArch, "INSTALL_HDR_PATH="+filepath.Join(TempDir, t.GoArch))
 	linuxMake.Dir = LinuxDir
 	if err := linuxMake.Run(); err != nil {
 		return err
 	}
 
+	buildDir := filepath.Join(TempDir, t.GoArch, "build")
 	// A Temporary build directory for glibc
-	if err := os.MkdirAll(BuildDir, os.ModePerm); err != nil {
+	if err := os.MkdirAll(buildDir, os.ModePerm); err != nil {
 		return err
 	}
-	defer os.RemoveAll(BuildDir)
+	defer os.RemoveAll(buildDir)
 
 	// Make the glibc headers we need for this architecture
 	confScript := filepath.Join(GlibcDir, "configure")
-	glibcConf := makeCommand(confScript, "--prefix="+TempDir, "--host="+t.GNUArch, "--enable-kernel="+MinKernel)
-	glibcConf.Dir = BuildDir
+	glibcArgs := []string{"--prefix=" + filepath.Join(TempDir, t.GoArch), "--host=" + t.GNUArch}
+	if t.LinuxArch == "loongarch" {
+		// The minimum version requirement of the Loongarch for the kernel in glibc
+		// is 5.19, if --enable-kernel is less than 5.19, glibc handles errors
+		glibcArgs = append(glibcArgs, "--enable-kernel=5.19.0")
+	} else {
+		glibcArgs = append(glibcArgs, "--enable-kernel="+MinKernel)
+	}
+	glibcConf := t.makeCommand(confScript, glibcArgs...)
+
+	glibcConf.Dir = buildDir
 	if err := glibcConf.Run(); err != nil {
 		return err
 	}
-	glibcMake := makeCommand("make", "install-headers")
-	glibcMake.Dir = BuildDir
+	glibcMake := t.makeCommand("make", "install-headers")
+	glibcMake.Dir = buildDir
 	if err := glibcMake.Run(); err != nil {
 		return err
 	}
 	// We only need an empty stubs file
-	stubsFile := filepath.Join(IncludeDir, "gnu/stubs.h")
+	stubsFile := filepath.Join(TempDir, t.GoArch, "include", "gnu", "stubs.h")
 	if file, err := os.Create(stubsFile); err != nil {
 		return err
 	} else {
@@ -374,19 +448,18 @@ func (t *target) makeHeaders() error {
 //
 // We generate C headers instead of a Go file, so as to enable references to the ABI from Cgo.
 func (t *target) makeABIHeaders() (err error) {
-	abiDir := filepath.Join(IncludeDir, "abi")
+	abiDir := filepath.Join(TempDir, t.GoArch, "include", "abi")
 	if err = os.Mkdir(abiDir, os.ModePerm); err != nil {
 		return err
 	}
 
-	cc := os.Getenv("CC")
-	if cc == "" {
+	if t.compiler == "" {
 		return errors.New("CC (compiler) env var not set")
 	}
 
 	// Build a sacrificial ELF file, to mine for C compiler behavior.
-	binPath := filepath.Join(TempDir, "tmp_abi.o")
-	bin, err := t.buildELF(cc, cCode, binPath)
+	binPath := filepath.Join(TempDir, t.GoArch, "tmp_abi.o")
+	bin, err := t.buildELF(t.compiler, cCode, binPath)
 	if err != nil {
 		return fmt.Errorf("cannot build ELF to analyze: %v", err)
 	}
@@ -414,9 +487,10 @@ func (t *target) makeABIHeaders() (err error) {
 func (t *target) buildELF(cc, src, path string) (*elf.File, error) {
 	// Compile the cCode source using the set compiler - we will need its .data section.
 	// Do not link the binary, so that we can find .data section offsets from the symbol values.
-	ccCmd := makeCommand(cc, "-o", path, "-gdwarf", "-x", "c", "-c", "-")
+	ccCmd := t.makeCommand(cc, "-o", path, "-gdwarf", "-x", "c", "-c", "-")
 	ccCmd.Stdin = strings.NewReader(src)
 	ccCmd.Stdout = os.Stdout
+	defer t.printAndResetBuilder()
 	if err := ccCmd.Run(); err != nil {
 		return nil, fmt.Errorf("compiler error: %v", err)
 	}
@@ -478,7 +552,7 @@ func (t *target) writeBitFieldMasks(bin *elf.File, out io.Writer) error {
 // makes the zsysnum_linux_$GOARCH.go file
 func (t *target) makeZSysnumFile() error {
 	zsysnumFile := fmt.Sprintf("zsysnum_linux_%s.go", t.GoArch)
-	unistdFile := filepath.Join(IncludeDir, "asm/unistd.h")
+	unistdFile := filepath.Join(TempDir, t.GoArch, "include", "asm", "unistd.h")
 
 	args := append(t.cFlags(), unistdFile)
 	return t.commandFormatOutput("gofmt", zsysnumFile, "mksysnum", args...)
@@ -495,22 +569,108 @@ func (t *target) makeZSyscallFile() error {
 	}
 
 	args := append(t.mksyscallFlags(), "-tags", "linux,"+t.GoArch,
-		"syscall_linux.go", archSyscallFile)
+		"syscall_linux.go",
+		archSyscallFile,
+	)
+
+	files, err := t.archMksyscallFiles()
+	if err != nil {
+		return fmt.Errorf("failed to check GOARCH-specific mksyscall files: %v", err)
+	}
+	args = append(args, files...)
+
 	return t.commandFormatOutput("gofmt", zsyscallFile, "mksyscall", args...)
 }
 
+// archMksyscallFiles produces additional file arguments to mksyscall if the
+// build constraints in those files match those defined for target.
+func (t *target) archMksyscallFiles() ([]string, error) {
+	// These input files don't fit the typical GOOS/GOARCH file name conventions
+	// but are included conditionally in the arguments to mksyscall based on
+	// whether or not the target matches the build constraints defined in each
+	// file.
+	//
+	// TODO(mdlayher): it should be possible to generalize this approach to work
+	// over all of syscall_linux_* rather than hard-coding a few special files.
+	// Investigate this.
+	inputs := []string{
+		// GOARCH: all except arm* and riscv.
+		"syscall_linux_alarm.go",
+	}
+
+	var outputs []string
+	for _, in := range inputs {
+		ok, err := t.matchesMksyscallFile(in)
+		if err != nil {
+			return nil, fmt.Errorf("failed to parse file %q: %v", in, err)
+		}
+		if ok {
+			// Constraints match, use for this target's code generation.
+			outputs = append(outputs, in)
+		}
+	}
+
+	return outputs, nil
+}
+
+// matchesMksyscallFile reports whether the input file contains constraints
+// which match those defined for target.
+func (t *target) matchesMksyscallFile(file string) (bool, error) {
+	f, err := os.Open(file)
+	if err != nil {
+		return false, err
+	}
+	defer f.Close()
+
+	var (
+		expr  constraint.Expr
+		found bool
+	)
+
+	s := bufio.NewScanner(f)
+	for s.Scan() {
+		// Keep scanning until a valid constraint is found or we hit EOF.
+		//
+		// This only supports single-line constraints such as the //go:build
+		// convention used in Go 1.17+. Because the old //+build convention
+		// (which may have multiple lines of build tags) is being deprecated,
+		// we don't bother looking for multi-line constraints.
+		if expr, err = constraint.Parse(s.Text()); err == nil {
+			found = true
+			break
+		}
+	}
+	if err := s.Err(); err != nil {
+		return false, err
+	}
+	if !found {
+		return false, errors.New("no build constraints found")
+	}
+
+	// Do the defined constraints match target's GOOS/GOARCH?
+	ok := expr.Eval(func(tag string) bool {
+		return tag == GOOS || tag == t.GoArch
+	})
+
+	return ok, nil
+}
+
 // makes the zerrors_linux_$GOARCH.go file
 func (t *target) makeZErrorsFile() error {
 	zerrorsFile := fmt.Sprintf("zerrors_linux_%s.go", t.GoArch)
-
-	return t.commandFormatOutput("gofmt", zerrorsFile, "./mkerrors.sh", t.cFlags()...)
+	return t.commandFormatOutput("gofmt2", zerrorsFile, "/"+filepath.Join("build", "unix", "mkerrors.sh"), t.cFlags()...)
 }
 
 // makes the ztypes_linux_$GOARCH.go file
 func (t *target) makeZTypesFile() error {
 	ztypesFile := fmt.Sprintf("ztypes_linux_%s.go", t.GoArch)
 
-	args := []string{"tool", "cgo", "-godefs", "--"}
+	cgoDir := filepath.Join(TempDir, t.GoArch, "cgo")
+	if err := os.MkdirAll(cgoDir, os.ModePerm); err != nil {
+		return err
+	}
+
+	args := []string{"tool", "cgo", "-godefs", "-objdir=" + cgoDir, "--"}
 	args = append(args, t.cFlags()...)
 	args = append(args, "linux/types.go")
 	return t.commandFormatOutput("mkpost", ztypesFile, "go", args...)
@@ -519,7 +679,7 @@ func (t *target) makeZTypesFile() error {
 // Flags that should be given to gcc and cgo for this target
 func (t *target) cFlags() []string {
 	// Compile statically to avoid cross-architecture dynamic linking.
-	flags := []string{"-Wall", "-Werror", "-static", "-I" + IncludeDir}
+	flags := []string{"-Wall", "-Werror", "-static", "-I" + filepath.Join(TempDir, t.GoArch, "include")}
 
 	// Architecture-specific flags
 	if t.SignedChar {
@@ -549,6 +709,26 @@ func (t *target) mksyscallFlags() (flags []string) {
 	return
 }
 
+// Merge all the generated files for Linux targets
+func mergeFiles() error {
+	// Setup environment variables
+	os.Setenv("GOOS", runtime.GOOS)
+	os.Setenv("GOARCH", runtime.GOARCH)
+
+	// Merge each of the four type of files
+	for _, ztyp := range []string{"zerrors", "zsyscall", "zsysnum", "ztypes"} {
+		cmd := exec.Command("go", "run", "./internal/mkmerge", "-out", fmt.Sprintf("%s_%s.go", ztyp, GOOS), fmt.Sprintf("%s_%s_*.go", ztyp, GOOS))
+		cmd.Stderr = os.Stderr
+		err := cmd.Run()
+		if err != nil {
+			return fmt.Errorf("could not merge %s files: %w", ztyp, err)
+		}
+		fmt.Printf("%s files merged\n", ztyp)
+	}
+
+	return nil
+}
+
 // generatePtracePair takes a pair of GOARCH values that can run each
 // other's binaries, such as 386 and amd64. It extracts the PtraceRegs
 // type for each one. It writes a new file defining the types
@@ -572,6 +752,7 @@ func generatePtracePair(arch1, arch2, archName string) error {
 	buf := bufio.NewWriter(f)
 	fmt.Fprintf(buf, "// Code generated by linux/mkall.go generatePtracePair(%q, %q). DO NOT EDIT.\n", arch1, arch2)
 	fmt.Fprintf(buf, "\n")
+	fmt.Fprintf(buf, "//go:build linux && (%s || %s)\n", arch1, arch2)
 	fmt.Fprintf(buf, "// +build linux\n")
 	fmt.Fprintf(buf, "// +build %s %s\n", arch1, arch2)
 	fmt.Fprintf(buf, "\n")
